/*
 *    Copyright (c) 2017 by Maciej Wiecierzewski
 */

#ifndef EVENTQUEUE_H
#define EVENTQUEUE_H

#include <functional>
#include <utility>
#include <queue>

#include "Event.h"
#include "EventEmitter.h"
#include "events.h"

namespace gui
{

//! GUISystem's internal event queue; collects fired events and affiliates them with particular event listener.

class EventQueue
{
    public:
        EventQueue() {};
        ~EventQueue() {};

        /** \brief Adds an event to the queue and affiliated event listener.
         *
         * \param event The event
         * \param eventListener EventListener handling the event
         *
         */
        void addToQueue(Event&& event, EventListener eventListener);

        /** \brief Is queue empty.
         *
         * \return Returns true if the queue is empty.
         *
         */
        bool empty();

        /** \brief Pops an event from the queue. The returned event is removed from the queue.
         *
         * \param eventListener Reference to EventListener handling the event.
         * \return Event
         *
         */
        Event getNextEvent(EventListener &eventListener);

    protected:

    private:

        //! EventQueue's inner class that contains an Event object and event listener it's destined.

        struct Entry
        {
            Event event;
            EventListener eventListener;

            Entry(Event& event, EventListener& eventListener) :
                event(std::move(event)),
                eventListener(eventListener)
            {
            }

            Entry(Entry&& other) :
                event(std::move(other.event)),
                eventListener(std::move(other.eventListener))
            {
            }

            Entry(Entry&) = delete;
        };

        std::queue<Entry> queue;
};

}

#endif // EVENTQUEUE_H
